"""
API Marketplace Client Library
"""

__version__ = "1.0.0.4"
__author__ = "venilo_o"
__email__ = "ilyasaminev3@mail.ru"

from .sync.client import EncarClient, KBchachachaClient
from .async_.client import AsyncEncarClient, AsyncKBchachachaClient
from .exceptions import MarketplaceAPIException, APIError, AuthError, ForbiddenError, ValidationError

__all__ = [
    # Sync clients
    "EncarClient",
    "KBchachachaClient",
    # Async clients
    "AsyncEncarClient",
    "AsyncKBchachachaClient",
    # Exceptions
    "MarketplaceAPIException",
    "APIError",
    "AuthError",
    "ForbiddenError",
    "ValidationError"
]



import pytest
from unittest.mock import Mock, patch
from api_marketplace_client import EncarClient, AuthError


class TestEncarClient:
    def setup_method(self):
        self.client = EncarClient(api_key="test_key")

    @patch('api_marketplace_client.client.requests.Session.request')
    def test_get_brands_success(self, mock_request):
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"brands": []}
        mock_request.return_value = mock_response

        result = self.client.get_brands()
        assert result == {"brands": []}

    @patch('api_marketplace_client.client.requests.Session.request')
    def test_auth_error(self, mock_request):
        mock_response = Mock()
        mock_response.status_code = 401
        mock_request.return_value = mock_response

        with pytest.raises(AuthError):
            self.client.get_brands()



"""Исключения для Marketplace API"""


class MarketplaceAPIException(Exception):
    """Базовое исключение для всех ошибок API"""
    pass


class APIError(MarketplaceAPIException):
    """Ошибка API запроса"""

    def __init__(self, message: str, status_code: int = None):
        super().__init__(message)
        self.status_code = status_code


class AuthError(MarketplaceAPIException):
    """Ошибка аутентификации"""
    pass


class ForbiddenError(MarketplaceAPIException):
    """Ошибка аутентификации"""
    pass


class ValidationError(MarketplaceAPIException):
    """Ошибка валидации данных"""
    pass



from typing import Optional, Dict, Any

import aiohttp
from aiohttp import ClientSession, ClientTimeout

from ..base import BaseClient, EncarMixin, KBchachachaMixin
from ..exceptions import APIError


class AsyncBaseAPIClient(BaseClient):
    def __init__(self, api_key: str, base_url: str = "https://api.autovenil.ru"):
        super().__init__(api_key, base_url)
        self._session: Optional[ClientSession] = None
        self.timeout = ClientTimeout(total=30)

    async def _get_session(self) -> ClientSession:
        if self._session is None or self._session.closed:
            connector = aiohttp.TCPConnector(
                limit=100,
                limit_per_host=30,
                ttl_dns_cache=300,
                use_dns_cache=True
            )
            self._session = ClientSession(
                headers=self.headers,
                timeout=self.timeout,
                connector=connector
            )

            return self._session

    async def _request(self, method: str, endpoint: str, **kwargs) -> Dict[str, Any]:
        url = self._build_url(endpoint)
        session = await self._get_session()

        try:
            async with session.request(method, url, **kwargs) as response:
                response_data = await response.json() if response.content_length else None
                self._handle_response_errors(response.status, response_data)
                return await response.json()

        except aiohttp.ClientError as e:
            raise APIError(f"Request failed: {str(e)}")

    async def close(self):
        if self._session and not self._session.closed:
            await self._session.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.close()


class AsyncEncarClient(AsyncBaseAPIClient, EncarMixin):
    """Клиент для работы с API Encar"""

    async def get_brands(self) -> Dict[str, Any]:
        """Получить список всех брендов"""
        return await self._request("GET", self._get_brands_endpoint())

    async def get_brand(self, brand_code: str) -> Dict[str, Any]:
        """Получить информацию о конкретном бренде"""
        return await self._request("GET", self._get_brand_endpoint(brand_code))

    async def get_models(self, brand_code: str) -> Dict[str, Any]:
        """Получить список всех моделей бренда"""
        return await self._request("GET", self._get_models_endpoint(brand_code))

    async def get_model(self, model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить информацию о конкретной модели"""
        params = {'brandCode': brand_code}
        return await self._request("GET", self._get_model_endpoint(model_code), params=params)

    async def get_generations(self, model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить список всех поколений модели"""
        params = {'brandCode': brand_code}
        return await self._request("GET", self._get_generations_endpoint(model_code), params=params)

    async def get_generation(self, generation_code: str, brand_code: str, model_code: str) -> Dict[str, Any]:
        """Получить информацию о конкретном поколении"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code
        }
        return await self._request("GET", self._get_generation_endpoint(generation_code), params=params)

    async def get_drive_types(self, generation_code: str, brand_code: str, model_code: str) -> Dict[str, Any]:
        """Получить список всех видов приводов поколения"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code
        }
        return await self._request("GET", self._get_drive_types_endpoint(generation_code), params=params)

    async def get_drive_type(
            self,
            drive_type_code: str,
            generation_code: str,
            model_code: str,
            brand_code: str
    ) -> Dict[str, Any]:
        """Получить информацию о конкретном виде привода"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code,
            'generationCode': generation_code
        }
        return await self._request("GET", self._get_drive_type_endpoint(drive_type_code), params=params)

    async def get_trims(
            self,
            drive_type_code: str,
            generation_code: str,
            model_code: str,
            brand_code: str
    ) -> Dict[str, Any]:
        """Получить список всех комплектаций"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code,
            'generationCode': generation_code
        }
        return await self._request("GET", self._get_trims_endpoint(drive_type_code), params=params)

    async def get_trim(
            self,
            trim_code: str,
            drive_type_code: str,
            generation_code: str,
            model_code: str,
            brand_code: str
    ) -> Dict[str, Any]:
        """Получить информацию о конкретной комплектации"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code,
            'generationCode': generation_code,
            'driveTypeCode': drive_type_code
        }
        return await self._request("GET", self._get_trim_endpoint(trim_code), params=params)

    async def get_fuel_types(
            self,
            brand_code: str,
            model_code: Optional[str] = None,
            generation_code: Optional[str] = None,
            drive_type_code: Optional[str] = None,
            trim_code: Optional[str] = None
    ) -> Dict[str, Any]:
        """Получить список всех видов топлива"""
        params = {'brandCode': brand_code}
        optional_params = {
            'modelCode': model_code,
            'generationCode': generation_code,
            'driveTypeCode': drive_type_code,
            'trimCode': trim_code
        }

        for key, value in optional_params.items():
            if value is not None:
                params[key] = value
        return await self._request("GET", self._get_fuel_types_endpoint(), params=params)

    async def get_fuel_type(
            self,
            fuel_type_code: str,
            brand_code: str,
            model_code: Optional[str] = None,
            generation_code: Optional[str] = None,
            drive_type_code: Optional[str] = None,
            trim_code: Optional[str] = None
    ) -> Dict[str, Any]:
        """Получить информацию о конкретном виде топлива"""
        params = {'brandCode': brand_code}

        optional_params = {
            'modelCode': model_code,
            'generationCode': generation_code,
            'driveTypeCode': drive_type_code,
            'trimCode': trim_code
        }

        for key, value in optional_params.items():
            if value is not None:
                params[key] = value

        return await self._request("GET", self._get_fuel_type_endpoint(fuel_type_code), params=params)

    async def get_cars(
            self,
            brand_code: str,
            page: int = 1,
            per_page: int = 20,
            model_code: Optional[str] = None,
            generation_code: Optional[str] = None,
            drive_type_code: Optional[str] = None,
            trim_code: Optional[str] = None,
            fuel_type_code: Optional[str] = None
    ) -> Dict[str, Any]:
        """Получить список автомобилей с пагинацией"""
        params = {
            'brandCode': brand_code,
            'page': page,
            'perPage': per_page
        }

        optional_params = {
            'modelCode': model_code,
            'generationCode': generation_code,
            'driveTypeCode': drive_type_code,
            'trimCode': trim_code,
            'fuelTypeCode': fuel_type_code
        }

        for key, value in optional_params.items():
            if value is not None:
                params[key] = value

        return await self._request("GET", self._get_cars_endpoint(), params=params)

    async def get_car(self, car_id: str | int) -> Dict[str, Any]:
        """Получить информацию об автомобиле по ID"""
        return await self._request("GET", self._get_car_endpoint(car_id))


class AsyncKBchachachaClient(AsyncBaseAPIClient, KBchachachaMixin):
    """Клиент для работы с API KBchachacha"""

    async def get_brands(self) -> Dict[str, Any]:
        """Получить список всех производителей"""
        return await self._request("GET", self._get_brands_endpoint())

    async def get_brand(self, brand_code: str) -> Dict[str, Any]:
        """Получить информацию о производителе"""
        return await self._request("GET", self._get_brand_endpoint(brand_code))

    async def get_models(self, brand_code: str) -> Dict[str, Any]:
        """Получить список моделей производителя"""
        return await self._request("GET", self._get_models_endpoint(brand_code))

    async def get_model(self, model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить информацию о модели"""
        params = {'brandCode': brand_code}
        return await self._request("GET", self._get_model_endpoint(model_code), params=params)

    async def get_generations(self, model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить список поколений модели"""
        params = {'brandCode': brand_code}
        return await self._request("GET", self._get_generations_endpoint(model_code), params=params)

    async def get_generation(self, generation_code: str, model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить информацию о поколении"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code
        }
        return await self._request("GET", self._get_generation_endpoint(generation_code), params=params)

    async def get_trims(self, generation_code: str,  model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить список модификаций поколения"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code
        }
        return await self._request("GET", self._get_trims_endpoint(generation_code), params=params)

    async def get_trim(self, trim_code: str, generation_code: str, model_code: str, brand_code: str) -> Dict[str, Any]:
        """Получить информацию о модификации"""
        params = {
            'brandCode': brand_code,
            'modelCode': model_code,
            'generationCode': generation_code
        }
        return await self._request("GET", self._get_trim_endpoint(trim_code), params=params)

    async def get_cars(
            self,
            brand_code: str,
            page: int = 1,
            per_page: int = 40,
            model_code: Optional[str] = None,
            generation_code: Optional[int] = None,
            trim_code: Optional[int] = None,
            grade_code: Optional[str] = None
    ) -> Dict[str, Any]:
        """Получить список автомобилей с фильтрацией"""
        params = {
            'brandCode': brand_code,
            'page': page,
            'perPage': per_page
        }

        optional_params = {
            'modelCode': model_code,
            'generationCode': generation_code,
            'trimCode': trim_code,
            'gradeCode': grade_code
        }

        for key, value in optional_params.items():
            if value is not None:
                params[key] = value

        return await self._request("GET", self._get_cars_endpoint(), params=params)

    async def get_car(self, car_id: str | int) -> Dict[str, Any]:
        """Получить детальную информацию об автомобиле"""
        return await self._request("GET", self._get_car_endpoint(car_id))



from abc import ABC, abstractmethod
from typing import Dict, Any

from exceptions import APIError, AuthError, ForbiddenError


class BaseClient(ABC):
    def __init__(self, api_key: str, base_url: str = "https://api.autovenil.ru"):
        self.api_key = api_key
        self.base_url = base_url
        self.headers = {
            "X-API-Key": self.api_key,
            "Content-Type": "application/json",
            "Accept": "application/json",
        }

    def _build_url(self, endpoint: str) -> str:
        return f"{self.base_url}{endpoint}"

    def _handle_response_errors(self, status_code: int, response_data: Dict[str, Any] = None):
        if status_code == 401:
            raise AuthError("Invalid API key or unauthorized access")
        if status_code == 403:
            raise ForbiddenError("Invalid API key or unauthorized access")
        if status_code >= 400:
            error_msg = "Unknown error occurred"
            if response_data:
                error_msg = response_data.get('detail', error_msg)
            raise APIError(f"API Error {status_code}: {error_msg}", status_code)

    @abstractmethod
    def _request(self, method: str, endpoint: str, **kwargs) -> Dict[str, Any]:
        pass


class EncarMixin:
    def _get_brands_endpoint(self) -> str:
        return '/v1/encar/brands/'

    def _get_brand_endpoint(self, brand_code: str) -> str:
        return f'/v1/encar/brands/{brand_code}'

    def _get_models_endpoint(self, brand_code: str) -> str:
        return f'/v1/encar/brands/{brand_code}/models'

    def _get_model_endpoint(self, model_code: str) -> str:
        return f'/v1/encar/models/{model_code}'

    def _get_generations_endpoint(self, model_code: str) -> str:
        return f'/v1/encar/models/{model_code}/generations'

    def _get_generation_endpoint(self, generation_code: str) -> str:
        return f'/v1/encar/generations/{generation_code}'

    def _get_drive_types_endpoint(self, generation_code: str) -> str:
        return f'/v1/encar/generations/{generation_code}/drive-types'

    def _get_drive_type_endpoint(self, drive_type_code: str) -> str:
        return f'/v1/encar/drive-types/{drive_type_code}'

    def _get_trims_endpoint(self, drive_type_code: str) -> str:
        return f'/v1/encar/drive-types/{drive_type_code}/trims'

    def _get_trim_endpoint(self, trim_code: str) -> str:
        return f'/v1/encar/trims/{trim_code}'

    def _get_fuel_types_endpoint(self) -> str:
        return '/v1/encar/fuel-types/'

    def _get_fuel_type_endpoint(self, fuel_type_code: str) -> str:
        return f'/v1/encar/fuel-types/{fuel_type_code}'

    def _get_cars_endpoint(self) -> str:
        return '/v1/encar/cars/'

    def _get_car_endpoint(self, car_id: str | int) -> str:
        return f'/v1/encar/cars/{car_id}'


class KBchachachaMixin:
    def _get_brands_endpoint(self) -> str:
        return '/v1/kb-chachacha/brands/'

    def _get_brand_endpoint(self, brand_code: str) -> str:
        return f'/v1/kb-chachacha/brands/{brand_code}'

    def _get_models_endpoint(self, brand_code: str) -> str:
        return f'/v1/kb-chachacha/brands/{brand_code}/models'

    def _get_model_endpoint(self, model_code: str) -> str:
        return f'/v1/kb-chachacha/models/{model_code}'

    def _get_generations_endpoint(self, model_code: str) -> str:
        return f'/v1/kb-chachacha/models/{model_code}/generations'

    def _get_generation_endpoint(self, generation_code: str) -> str:
        return f'/v1/kb-chachacha/generations/{generation_code}'

    def _get_trims_endpoint(self, generation_code: str) -> str:
        return f'/v1/kb-chachacha/generations/{generation_code}/trims'

    def _get_trim_endpoint(self, trim_code: str) -> str:
        return f'/v1/kb-chachacha/trims/{trim_code}'

    def _get_cars_endpoint(self) -> str:
        return '/v1/kb-chachacha/cars/'

    def _get_car_endpoint(self, car_id: str | int) -> str:
        return f'/v1/kb-chachacha/cars/{car_id}'



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = [line.strip() for line in fh if line.strip() and not line.startswith("#")]

setup(
    name="api-marketplace-client",
    version="1.0.0.4",
    license="MIT",
    author="Ilyas Aminev",
    author_email="ilyasaminev3@mail.ru",
    description="Python client for API Marketplace (Encar & KBchachacha)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/venilo/api-marketplace-client.git",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    python_requires=">=3.8",
    install_requires=requirements,
    extras_require={
        "dev": [
            "pytest>=6.0",
            "pytest-cov>=2.0",
            "black>=21.0",
            "flake8>=3.8",
            "mypy>=0.800",
        ],
    },
    keywords="api marketplace client encar kbchachacha cars automotive",
)
